﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace UtiliKey
{
    static class Morse
    {
        #region Codes
        public static Dictionary<char, string> Codes { get { return _codes; } }

        private static Dictionary<char, string> _codes = new Dictionary<char, string>
        {
            { 'a', "._" },
            { 'b', "_..." },
            { 'c', "_._." },
            { 'd', "_.." },
            { 'e', "." },
            { 'f', ".._." },
            { 'g', "__." },
            { 'h', "...." },
            { 'i', ".." },
            { 'j', ".___" },
            { 'k', "_._" },
            { 'l', "._.." },
            { 'm', "__" },
            { 'n', "_." },
            { 'o', "___" },
            { 'p', ".__." },
            { 'q', "__._" },
            { 'r', "._." },
            { 's', "..." },
            { 't', "_" },
            { 'u', ".._" },
            { 'v', "..._" },
            { 'w', ".__" },
            { 'x', "_.._" },
            { 'y', "_.__" },
            { 'z', "__.." },
            { '0', "_____" },
            { '1', ".____" },
            { '2', "..___" },
            { '3', "...__" },
            { '4', "...._" },
            { '5', "....." },
            { '6', "_...." },
            { '7', "__..." },
            { '8', "___.." },
            { '9', "____." },
            { '.', "._._._" },
            { ',', "__..__" },
            { '?', "..__.." },
            { '\'', ".____." },
            { '!', "!" },
            { '/', "_.._." },
            { '(', "_.__." },
            { ')', "_.__._" },
            { '&', "._..." },
            { ':', "___..." },
            { ';', "_._._." },
            { '=', "_..._" },
            { '+', "._._." },
            { '-', "_...._" },
            { '_', "..__._" },
            { '"', ".__..__." },
            { '$', "..._.._" },
            { '@', ".__._" },
            { ' ', "/" },
            { '\n', "\n" },
            { '\r', "\r" }
        };
        #endregion

        public static string MorseEnc(string str)
        {
            char[] inChr = str.ToLower().ToCharArray();

            List<string> outChar = new List<string>();

            for (int i = 0; i < inChr.Length; i++)
            {
                if (Codes.ContainsKey(inChr[i]))
                    outChar.Add(Codes[inChr[i]]);
                else outChar.Add(inChr[i].ToString());
            }

            string outString = String.Join(" ", outChar);

            while (outString.Contains("! !"))
                outString = outString.Replace("! !", "!!");
            while (outString.Contains(" \n "))
                outString = outString.Replace(" \n ", "\n");

            return outString;
        }

        public static string MorseDec(string str)
        {
            while (str.Contains('-'))
                str = str.Replace('-', '_');

            string[] inStrings = str.Split(' ', '\n');
            string outString = "";

            foreach (string word in inStrings)
            {
                if (!Codes.ContainsValue(word))
                {
                    outString += word;
                    continue;
                }

                foreach (KeyValuePair<char, string> pair in Codes)
                    if (word == pair.Value)
                    {
                        outString += pair.Key;
                        break;
                    }
            }

            return outString;
        }

        public static string XForm(string str, bool dec)
        {
            if (dec) return MorseDec(str);
            return MorseEnc(str);
        }
    }
}
