﻿using System;
using System.Linq;
using System.Text;
using System.Security.Cryptography;

namespace UtiliKey
{
    class Transforms
    {
        public Transforms() { }

        #region strEncoding
        private Encoding _encoding = Encoding.ASCII;

        public Encoding strEncoding
        {
            get { return _encoding; }
            set { _encoding = value; }
        }
        #endregion

        #region Rot13
        public static string Rot13(string input, bool discard)
        {
            char[] cArray = input.ToCharArray();

            for (int i = 0; i < cArray.Length; i++)
            {
                int number = (int)cArray[i];

                if (number >= 'a' && number <= 'z')
                {
                    if (number > 'm')
                        number -= 13;
                    else number += 13;
                }
                else if (number >= 'A' && number <= 'Z')
                {
                    if (number > 'M') number -= 13;
                    else number += 13;
                }
                cArray[i] = (char)number;
            }
            return new string(cArray);
        }
        #endregion

        #region Base64
        public string Base64(string str, bool decode)
        {
            if (decode)
            {
                byte[] decbuff = Convert.FromBase64String(str);
                return strEncoding.GetString(decbuff);
            }
            byte[] encbuff = strEncoding.GetBytes(str);
            return Convert.ToBase64String(encbuff);
        }
        #endregion

        #region Byte Encoding
        public string Binary(string str, bool decode)
        {
            return CodeBase(str, decode, 2, false);
        }

        public string Octal(string str, bool decode)
        {
            return CodeBase(str, decode, 8, false);
        }

        public string Decimal(string str, bool decode)
        {
            return CodeBase(str, decode, 10, false);
        }

        public string Hex(string str, bool decode)
        {
            return CodeBase(str, decode, 16, false);
        }
        #endregion

        #region Full-Byte Encoding
        public string BinaryFull(string str, bool decode)
        {
            return CodeBase(str, decode, 2, true);
        }

        public string OctalFull(string str, bool decode)
        {
            return CodeBase(str, decode, 8, true);
        }

        public string DecimalFull(string str, bool decode)
        {
            return CodeBase(str, decode, 10, true);
        }

        public string HexFull(string str, bool decode)
        {
            return CodeBase(str, decode, 16, true);
        }
        #endregion

        #region Main Encoding
        private const char joiner = ' ';

        private string CodeBase(string str, bool decode, int basis,
            bool fullByte)
        {
            if (fullByte) return decode ?
                DecodeBaseFull(str, basis) :
                EncodeBaseFull(str, basis);
            return decode ?
                DecodeBase(str, basis) :
                EncodeBase(str, basis);
        }

        private static string[] splitString(string str, int maxLen)
        {
            //If it contains the joiner-character, splitting is easy.
            if (str.Contains(joiner) ||
                str.Length == maxLen ||
                str.Length == maxLen * 2)
                return str.Trim(joiner).Split(joiner);

            //Otherwise, we have to divide it by enumerating through it!
            return Enumerable.Range(0, str.Length / maxLen).
                    Select(i => str.Substring(i * maxLen, maxLen)).ToArray<string>();
        }

        private string EncodeBase(string str, int basis)
        {
            int maxLen = Convert.ToString(byte.MaxValue, basis).Length;

            byte[] theBytes = strEncoding.GetBytes(str);

            string[] theStrings = new string[theBytes.Length];

            for (int i = 0; i < theBytes.Length; i++)
                theStrings[i] = Convert.ToString(theBytes[i],
                    basis).PadLeft(maxLen, '0');

            return String.Join(joiner.ToString(), theStrings);
        }

        private string DecodeBase(string str, int basis)
        {
            int maxLen = Convert.ToString(byte.MaxValue, basis).Length;

            string[] theStrings = splitString(str, maxLen);

            byte[] theBytes = new Byte[theStrings.Length];

            //Iterate through the strings, 
            for (int i = 0; i < theStrings.Length; i++)
            {
                if (theStrings[i] == null || theStrings[i] == string.Empty
                    || theStrings[i] == "") continue;
                theBytes[i] = (byte)Convert.ToInt32(theStrings[i], basis);
            }
            return strEncoding.GetString(theBytes);
        }

        private static string EncodeBaseFull(string str, int basis)
        {
            int maxLen = Convert.ToString(byte.MaxValue, basis).Length;
            char[] theChars = str.ToCharArray();
            string[] theStrings = new string[theChars.Length];

            for (int i = 0; i < theChars.Length; i++)
            {
                int curChar = theChars[i];
                theStrings[i] = Convert.ToString(curChar, basis)
                    .PadLeft(curChar > (byte.MaxValue / 2) ? maxLen * 2 : maxLen, '0');
            }
            return String.Join(joiner.ToString(), theStrings);
        }

        private static string DecodeBaseFull(string str, int basis)
        {
            int maxLen = Convert.ToString(byte.MaxValue, basis).Length;

            string[] theStrings = splitString(str, maxLen);

            string retString = "";

            foreach (string curStr in theStrings)
            {
                int curChar = Convert.ToInt32(curStr, basis);
                retString += (char)curChar;
            }
            return retString;
        }
        #endregion

        #region MD5 Encoding
        public string MD5Encode(string value, bool discard)
        {
            MD5 coder = MD5.Create();

            byte[] buffer = strEncoding.GetBytes(value);

            byte[] encoded = coder.ComputeHash(buffer);

            return BitConverter.ToString(encoded).Replace("-", "").ToLower();
        }
        #endregion
    }
}
