﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace UtiliKey
{
    public partial class frmMain : Form
    {
        Transforms TFs = new Transforms();

        Command[] commands;

        public frmMain()
        {
            InitializeComponent();

            commands = new Command[]
            {
                new Command("Rot13", Transforms.Rot13, true),
                new Command("Morse", Morse.XForm, false),
                new Command("URL (normal)", HtmlTransforms.UrlSafe, false),
                new Command("URL (path-safe)", HtmlTransforms.UrlNormal, false),
                new Command("URL (every character)", HtmlTransforms.UrlAllChars, false),
                new Command("HTML Entities", HtmlTransforms.HTMLEntities, false),
                new Command("Base64", TFs.Base64),
                new Command("MD5 Hash", TFs.MD5Encode, true, true),
                new Command("Fullwidth Characters", WidthTransforms.Convert, false, false),
                new Command("Binary", TFs.Binary),
                new Command("Octal", TFs.Octal),
                new Command("Decimal", TFs.Decimal),
                new Command("Hexadecimal", TFs.Hex),
                new Command("Binary (full bytes)", TFs.BinaryFull, false),
                new Command("Octal (full bytes)", TFs.OctalFull, false),
                new Command("Decimal (full bytes)", TFs.DecimalFull, false),
                new Command("Hexadecimal (full bytes)", TFs.HexFull, false),
            };

            comOptions.DataSource = commands;
            
            //Can't do autocomplete in Mono, b'duh
#if AUTOCOMP
            AutoCompleteStringCollection autoComp = new AutoCompleteStringCollection();
            foreach (var curItem in commands)
            {
                autoComp.Add(curItem.ToString());
            }
            comOptions.AutoCompleteCustomSource = autoComp;
            comOptions.AutoCompleteMode = AutoCompleteMode.SuggestAppend;
            comOptions.AutoCompleteSource = AutoCompleteSource.CustomSource;
            oldText = comOptions.Text;
#endif
        }

        private void comOptions_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnDecode.Enabled = !commands[comOptions.SelectedIndex].OneWay;
            chkUnicode.Enabled = commands[comOptions.SelectedIndex].UnicodeAllowed;
            if (!chkUnicode.Enabled)
            {
                chkUnicode.Checked = false;
                TFs.strEncoding = Encoding.ASCII;
            }
        }

        private void btnEncode_Click(object sender, EventArgs e)
        {
            EncodeAct(false);
        }

        private void btnDecode_Click(object sender, EventArgs e)
        {
            EncodeAct(true);
        }

        private void EncodeAct(bool decode)
        {
#if !DEBUG
            try
            {
#endif
                string output =
                    commands[comOptions.SelectedIndex].Action(textInput.Text,
                    decode);
                textOutput.Text = output;
#if !DEBUG
            }
            catch (Exception e)
            {
                string de = decode ? "de" : "en";
                MessageBox.Show(e.Message, "Error attempting to " +
                    de + "code " + commands[comOptions.SelectedIndex].ToString());
            }
#endif
        }

        private void keyCopy(object sender, KeyEventArgs e)
        {
            if (!e.Control) return;
            TextBox snd = (TextBox)sender;
            if (e.KeyCode == Keys.A)
                snd.SelectAll();
            if (e.KeyCode == Keys.C)
                snd.Copy();
        }

        private void btnSwapUp_Click(object sender, EventArgs e)
        {
            textInput.Text = textOutput.Text;
            textOutput.Text = null;
        }

        #region Text Changed
        private void textInput_TextChanged(object sender, EventArgs e)
        {
            lblInput.Text = textInput.Text.Length + "";
        }

        private void textOutput_TextChanged(object sender, EventArgs e)
        {
            lblOutput.Text = textOutput.Text.Length + "";
        }

        private void chkUnicode_CheckedChanged(object sender, EventArgs e)
        {
            if (chkUnicode.Checked)
                TFs.strEncoding = Encoding.UTF8;
            else TFs.strEncoding = Encoding.ASCII;
        }
        #endregion

        #region Combobox Changed
        string oldText;

        private void comOptions_Validating(object sender, CancelEventArgs e)
        {
            int dex = Array.FindIndex<Command>(commands, x => comOptions.Text.Equals(
                x.ToString(), StringComparison.InvariantCultureIgnoreCase));

            if (dex == -1)
            {
                comOptions.Text = oldText;
                //e.Cancel = true;
            }
            else
            {
                oldText = commands[dex].ToString();
                comOptions.SelectedIndex = dex;
                if (comOptions.Text != oldText) comOptions.Text = oldText;
            }
        }
        #endregion
    }
}
